﻿if (([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator")) {
    #Requires -Version 4.0
    #Requires #-RunAsAdministrator

    Write-Output "Registering ELHC repository (ProGet)"
    
    if ((Get-PSRepository -ErrorAction SilentlyContinue -Name ELHCRepo) -eq $null) {
        Register-PSRepository -Name ELHCRepo -SourceLocation https://package.evidencelibrary.com/nuget/ELHealthcheck/ -InstallationPolicy Trusted
    }
    else {
        Write-Output "------Repo already registered"
    }
    Write-Output "Installing ELHC module..."
    
    if ((Get-Module -ErrorAction SilentlyContinue -Name ELHealthCheck) -eq $null) {
        Find-Module -Name ELHealthCheck -Repository ELHCRepo | Install-Module | Import-Module
    }
    else {
        Write-Output "------ELHealthcheck already installed"
    }

    $version = (Find-Module -Name ELHealthcheck -Repository ELHCRepo).version.tostring()
    ELHealthcheck\New-ELHC -version $version -timeout 5
    ELHealthcheck\Set-ELHCConfig

    Write-Output "Now we will make ELHealthcheck run as a scheduled task."

    if ([environment]::OSVersion.Version -ge (New-Object 'Version' 6,2)) {
        $action = New-ScheduledTaskAction -Execute powershell.exe -Argument "-executionpolicy bypass -command &{invoke-elhc -verbose}"
        $trigger = New-ScheduledTaskTrigger -Daily -At 5am -RandomDelay (New-TimeSpan -Minutes 30)
        $principal = New-ScheduledTaskPrincipal -UserId $env:USERNAME -RunLevel Highest
        Register-ScheduledTask -Action $action -Trigger $trigger -TaskName ELHealthcheck -Description "Runs Evidence Library Healthcheck daily" -Principal $principal
    }

    else {
        write-warning "Powershell scheduled tasks is not supported on your version of Windows"
    }

    Write-Output "Check your scheduled tasks.  Execute ELHealthcheck from there to ensure proper installation."
    Start-Sleep -Seconds 5 
    
    
}
else {
    Write-Warning "Attempting to elevate..."

    Start-Process powershell -Verb runas -ArgumentList "-file $PSScriptRoot\$($MyInvocation.MyCommand)"
    exit
}